/* $Id: limits.c,v 2.11 1997/01/02 16:46:27 jt Exp $ */

#include <stdio.h>
#include <math.h>

#include "xyplot.h"

extern int verbose;

#define vshowlim(p) if (verbose)\
	fprintf(stderr,". limits: %g %g %g %g\n",p->xmult,p->xlb,p->xub,p->xquant)

struct z {
    float lb, ub, mult, quant;
};

static int countmark __P((struct xy *));
static void setloglim __P((struct z *, int, int, float, float));
static void setlinlim __P((struct z *, int, int, float, float));
 
#define fswap(a,b) do { float t; t=a; a=b; b=t; } while(0)
#define iswap(a,b) do { int t; t=a; a=b; b=t; }   while(0)
 
int
setlim(p)
struct xy *p;
{
    float           t, delta, sign;
    struct z        z;
    int             lmark[512], mark[512];
    float           lb, ub;
    int             lbf, ubf;
	
    lb = p->xlb;
    ub = p->xub;
    delta = ub - lb;
    if (p->xqf) {               /* user supplied quant */
        if (delta * p->xquant <= 0) {
            fprintf(stderr, "%e %e %e", p->xlb, p->xub, p->xquant);
            fatal("User-supplied axis limits error");
        }
        /* return; */
    }
    sign = 1;
    lbf = p->xlbf;              /* is user supplied lower bound */
    ubf = p->xubf;              /* is user supplied upper bound */
    if (delta < 0) {
        sign = -1;
        fswap(lb,ub);
	iswap(lbf,ubf);
    } else if (delta == 0) {
        if (ub > 0) {
            ub = 2 * ub;
            lb = 0;
        } else if (lb < 0) {
            lb = 2 * lb;
            ub = 0;
        } else {
            ub = 1;
            lb = -1;
        }
    }
    vshowlim(p);

#if 0 /* this logic doesnt work */
    /* if negative range is specified for log axes, then assume that
     * linear scaling is what is really desired */
    if (p->xf == log10 && ( ub<0 || lb<0)) {
	p->xf = ident;
	p->xlogf = 0;
    }
#endif
    if (p->xf == log10 && ( ub<0 || lb<0)) {
	fprintf(stderr,"Warning: invalid range\n");
	ub = fabs(ub);
	lb = fabs(lb);
	if (ub < lb)
	    fswap(ub,lb);
    }
    /* if p->xlogf says it wants to be a log-like in its labeling; but
     * if the range is less than 2.0, then override that and make it
     * linear-like in its labeling (but still use log for the actual
     * placement of the axes */
    if (p->xlogf && ub < 2.0*lb)
	p->xlogf = 0;

    if (p->xlogf) {
	if (verbose) fprintf(stderr,"log-like labeling\n");
	vshowlim(p);
        setloglim(&z,lbf, ubf, lb, ub);
        p->xlb = z.lb;
        p->xub = z.ub;
        p->xmult *= z.mult;
	p->xllb = z.quant;
        if (!p->xqf)
	    p->xquant = 10;
        if (p->xquant < 1)
            p->xquant = 1;

    } else {

	vshowlim(p);
	setlinlim(&z, lbf, ubf, lb, ub);
	if (sign > 0) {
	    p->xlb = z.lb;
	    p->xub = z.ub;
	} else {
	    p->xlb = z.ub;
	    p->xub = z.lb;
	}
	vshowlim(p);
	p->xmult *= z.mult;
	vshowlim(p);
	if (! p->xqf)
	    p->xquant = sign * z.quant;
	else
	    p->xquant *= z.mult;
    }	
    vshowlim(p);
}
 
static void
setloglim(v,lbf, ubf, lb, ub)
struct z *v;
int lbf, ubf; 
float lb, ub;
{
    float           r, s, t;
    
    if (verbose) fprintf(stderr,	
			 "setloglim: (%d)%g, (%d)%g\n",
			 lbf,lb,ubf,ub);						 
    
    for (s = 1; lb * s < 1; s *= 10)
	;/* s = 10^k such that lb < 1/s */
    lb *= s;
    ub *= s;
    /* if lb was less than 10, now 1 <= lb < 10; 
     *             and ub is similarly rescaled 
     * otherwise, lb is unchanged
     */
    for (r = 1; 10 * r <= lb; r *= 10)
	;
    /* r chosen so that: 1 <= lb/r < 10;
     * nb: r=1 unless original lb was greater than 10
     */
    for (t = 1; t < ub; t *= 10)
	;
    
    if (verbose) fprintf(stderr,	
			 "...loglim: lb=%g, ub=%g, s=%g, r=%g\n",
			 lb, ub, s, r);
    
    v->lb = !lbf ? r : lb;
    v->ub = !ubf ? t : ub;
    
    if (verbose) fprintf(stderr,	
			 "...loglim: v->lb=%g, v->ub=%g\n",
			 v->lb, v->ub, s, r);

    /* We'll need to revisit this logic! */
    if (ub / lb < SHORTLOGRANGE) {
        if (!lbf) {
            if (lb >= 5 * v->lb)
                v->lb *= 5;
            else if (lb >= 2 * v->lb)
                v->lb *= 2;
        }
        if (!ubf) {
            if (ub * 5 <= v->ub)
                v->ub /= 5;
            else if (ub * 2 <= v->ub)
                v->ub /= 2;
        }
    }
    v->mult = s;
    v->quant = r;
}
static double
tentothe(int e)
{
    double t=1.0;
    while (e > 0) {
        t *= 10.0;
        --e;
    }
    while (e < 0) {
        t /= 10.0;
        ++e;
    }
    return t;
}

static void
setlinlim(v,lbf, ubf, xlb, xub)
struct z        *v;
int             lbf, ubf;
float           xlb, xub;
{
    int sgn=1;
    double coef,prec;
    double xllb,xuub,lb,ub;
    double minquant;
    
    if (xlb > xub) {
        double tmp; tmp = xub; xub = xlb; xlb = tmp; sgn=-1;
    }
    if (xlb == xub) {
        if (xlb == 0) {
            xlb = xllb = -1;
            xub = xuub =  1;
        } else {
            xllb = xlb*0.8999;
            xuub = xub*1.1001;
            xlb = xlb*0.99;
            xub = xub*1.01;
        }
    } else {
    
        /* choose denominator 7.999 because it is the largest number
           such that the range 1-9 gets converted to 0-10 */
        xllb = xlb - (xub-xlb)/7.999;
        xuub = xub + (xub-xlb)/7.999;
    }
    

    if (verbose)
        fprintf(stderr,"xllb,xlb,xub,xuub=[%g,%g]-[%g,%g]\n",xllb,xlb,xub,xuub);

    prec = tentothe((int)log10(xuub-xllb)+1);
    coef = 2;
    do {
        if (coef == 1) {
            prec = prec/10;
            coef = 5;
        } else if (coef == 5) {
            coef = 2;
        } else if (coef == 2) {
            coef = 1;
        }
        lb = coef*prec*floor(xlb/(coef*prec));
        ub = coef*prec*ceil(xub/(coef*prec));
        if (verbose>1)
            fprintf(stderr,"preliminary range=[%g,%g]\n",lb,ub);
    } while(!((lb > xllb) && (ub < xuub)));

    if (sgn<0) {
        double tmp; tmp=lb; lb=ub; ub=tmp;
    }
        
    if (verbose)
        fprintf(stderr,"range=[%g,%g]\n",lb,ub);

    v->ub = ubf ? xub : ub;
    v->lb = lbf ? xlb : lb;
    v->mult = 1.0;

    minquant = fabs(ub - lb)/6;
    prec = tentothe((int)floor(log10(minquant)));
    if (prec >= minquant) {
        v->quant = prec;
    } else if (2*prec >= minquant) {
        v->quant = 2*prec;
    } else if (5*prec >= minquant) {
        v->quant = 5*prec;
    } else if (10*prec >= minquant) {
        v->quant = 10*prec;
    }
    if (verbose)
        fprintf(stderr,"[%g,%g],prec=%g,quant=%g\n",lb,ub,prec,v->quant);
    
}

/* countmark: counts marks completely inside range: thus,
   if range is 1 -> 100, only 10 is completely inside, so
   returns a value of 1 */
static int countmark(p)
struct xy *p;
{
    float x;
    int xn=0;
    for (x = p->xllb; x <= p->xub; x *= 10) {
	if ( in_logrange(x,p,EPS) ) 
	    ++xn;
    }
    return xn;
}

int
in_range(x,p,eps)
float x, eps;
struct xy *p;
{
    float delx;
    delx = fabs(p->xub - p->xlb);
    if ((p->xquant>0 && (x< p->xlb -eps*delx || x> p->xub +eps*delx))
	||(p->xquant<0 && (x< p->xub -eps*delx || x> p->xlb +eps*delx)))
	return NO;
    else
	return YES;
}
int 
in_logrange(x,p,eps)
float x, eps;
struct xy *p;
{
    return ((1.0+eps) * p->xlb < x && (1.0-eps) * p->xub > x) ;
}
void 
outer_range(lo,hi,p)
float *lo,*hi;
struct xy *p;
{
    float q,xl,xu;
	
    q = p->xquant;
	
    if (q > 0) {
	*lo = modfloor(p->xlb - q / 6, q);
	*hi = modceil( p->xub + q / 6, q);
    } else {
	*lo = modfloor(p->xub + q / 6, -q);
	*hi = modceil( p->xlb - q / 6, -q);
    }
}

