/* $Id: xyplot.h,v 2.7 1997/11/26 04:18:37 rosalia Exp $ */
#ifndef _XYPLOT_H_
#define _XYPLOT_H_

#define YES (1)
#define NO  (0)
 
/* normally we might use HUGE here, but for Linux at least, HUGE is
 * the largest double, and what we want is the largest float.  in a
 * future release of xyplot, we'll want the largest double; in that
 * case we can go back to defining VERYBIG to being HUGE
 */
#ifndef VERYBIG
#ifdef FLOATMAX
#define VERYBIG FLOATMAX
#else
#define VERYBIG 1e30
#endif /* FLOATMAX */
#endif /* VERYBIG */

#define EPS           1e-5          /* tolerance factor */
#define SHORTLOGRANGE 100           /* if log range is less than this, then label
				     * with 2's and 5's */
#define BSIZ 256                    /* default BufferSize */


#define SIZE 4096                   /* resolution of a full "page" */
#define TSIZE (0.0125*SIZE)         /* convenient "units" for small features 
				     * on the graph; it is also the default 
				     * size of a subtick
				     */
#define SSize (TSIZE*overallsize)   /* convenient "scaled" units */

#define TitleSize SSize * titlesize
#define titleSize titlesize*overallsize*SIZE/4096
#define xTitleSize SSize * xtitlesize
#define xtitleSize xtitlesize*overallsize*SIZE/4096
#define yTitleSize SSize * ytitlesize
#define ytitleSize ytitlesize*overallsize*SIZE/4096
#define NumlblSize SSize * numlblsize
#define numlblSize numlblsize*overallsize*SIZE/4096

/* defaults */
 
#define OVERALLSIZE 1.0
#define TICKFACTOR 1.0
#define TICKSPERTICK 5
#define XTICKSPERTICK 5
#define YTICKSPERTICK 5
#define AXLWIDTH 1.0
#define LINWIDTH 0.0
#define LBLWIDTH 1.0
#define SYMSIZE 1.5
#define NUMLBLSIZE 1.5
#define TITLESIZE 1.0
#define XTITLESIZE 1.5
#define YTITLESIZE 1.5
#define GRIDSTYLE 1
#define GRIDLINESTYLE 2
#define MODE 1
#define COLOUR 0
#define COLMOD 10
 
extern char *colours[];
extern int colour;
extern int global_colour;
extern int ncolours;

/*-- structures:
 *   xy:        x or y axis information
 *   datum:     single data point
 *   datastyle: how to plot a data point
 */
struct xy {
    int             xlbf, xubf; /* flag:explicit lower and upper bound */
    int             xqf;        /* flag:explicit quantum */
    double          (*xf) ();   /* transform function, e.g. log */
    float           xa, xb;     /* scaling coefficients */
    float           xlb, xub;   /* lower and upper bound */
    float           xllb, xuub; /* lower-lower and upper-upper bound */
    int 	    xlogf;      /* log-style axes */
    float           xquant;     /* quantum */
    float           xoff;       /* screen offset fraction */
    float           xsize;      /* screen fraction */
    int             xbot, xtop; /* screen coords of border */
    float           xmult;      /* scaling constant */
};

struct datum {
    float           xv;     /* x coordinate */
    float           yv;     /* y coordinate */
    float           yerr;   /* optional error on y */
    struct 
	datastyle  *style;
    
};
struct datastyle {
    char            line_mode;  /* current line mode */
    char            line_colour;/* current line color: line added DMD 8/2/95 */
    char            err_flag;   /* does data have an error bar */
    char            err_xy;     /* is errorbar on x or y axis? */
    char            con_flag;   /* conect to previous point ? */
    char           *lblptr;     /* optional plot symbol or label */
    float	    symsiz;	/* the size of the symbol or label */
    char            symb_flag;  /* is *lblptr a symbol or a label */
    float	    line_width; /* line width */
    struct
	datastyle  *parent;     /* housekeeping */
};

/* neg(x): returns abs(x) if x negative, 0 otherwise */
/* pos(x): returns abs(x) if x positive, 0 otherwise */
#define neg(size)       (((size)>0)? 0 : -(size))
#define pos(size)       (((size)>0)? (size) : 0 )
#define	max(a,b)	((a)>(b)?(a):(b))

#define fatal(errstring)  fprintf(stderr,"%s[%d]: %s\n",\
				  __FILE__,__LINE__,errstring),exit(-1)

/*** PROTOTYPES ***/

#ifndef __P
# ifdef __STDC__
#  define __P(x) x
# else
#  define __P(x) ()
# endif /* __STDC__ */
#endif /* __P */

double ident __P((double));

void axes __P((void));
void axlab __P((char, struct xy *));
void fatal __P((char *));
int gs_axes __P((char *));
int in_range __P((float, struct xy *, float));
int in_logrange __P((float, struct xy *, float));
float modceil __P((float, float));
float modfloor __P((float, float));
void newm __P((int));
void newc __P((int));
void neww __P((float));
int num_colours __P((void));
void parse __P((char *, char *));
int setlim __P((struct xy *));
int setmark __P((struct xy *, int));
void setticksize __P((void));
void title __P((void));
void xyticks __P((char, int, int));

/** relics... */ 
int utou __P((char *));
void gtou __P((char *, char *));

/** Plot Commands **/
void move __P((int,int));
void point __P((int,int));
void line __P((int,int,int,int));
void cont __P((int,int));
void space __P((int,int,int,int));
void label __P((char *));
void linemod __P((char *));
void linecol __P((char *));
void erase __P((void));
void width __P((float));
void ascii __P((char *, int, int, float));
void rascii __P((char *, int, int, float));
void cascii __P((char *, int, int, float));
void cvascii __P((char *, int, int, float));

char *index __P((char *, char));


#endif /* define _XYPLOT_H_ */

